@@ -181,13 +181,13 @@ xfdesktop_settings_do_single_preview(GtkTreeModel *model,
     /* If we didn't create a thumbnail there might not be a thumbnailer service
      * or it may not support that format */
     if(thumbnail == NULL) {
-        pix = exo_gdk_pixbuf_new_from_file_at_max_size(filename,
-                                                       PREVIEW_WIDTH, PREVIEW_HEIGHT,
-                                                       TRUE, NULL);
+        pix = gdk_pixbuf_new_from_file_at_scale(filename,
+                                                PREVIEW_WIDTH, PREVIEW_HEIGHT,
+                                                TRUE, NULL);
     } else {
-        pix = exo_gdk_pixbuf_new_from_file_at_max_size(thumbnail,
-                                                       PREVIEW_WIDTH, PREVIEW_HEIGHT,
-                                                       TRUE, NULL);
+        pix = gdk_pixbuf_new_from_file_at_scale(thumbnail,
+                                                PREVIEW_WIDTH, PREVIEW_HEIGHT,
+                                                TRUE, NULL);
         g_free(thumbnail);
     }
 
@@ -1276,12 +1276,12 @@ xfdesktop_settings_background_tab_change_bindings(AppearancePanel *panel,
         /* If the first color doesn't exist, try to load the old one */
         if(!xfconf_channel_has_property(channel, buf)) {
             GValue value = { 0, };
-            old_property = xfdesktop_settings_generate_old_binding_string(panel, "color1");
+            old_property = xfdesktop_settings_generate_old_binding_string(panel, "color2");
 
             xfconf_channel_get_property(channel, old_property, &value);
 
             if(G_VALUE_HOLDS_BOXED(&value)) {
-                gtk_color_button_set_color(GTK_COLOR_BUTTON(panel->color1_btn),
+                gtk_color_button_set_color(GTK_COLOR_BUTTON(panel->color2_btn),
                                            g_value_get_boxed(&value));
                 g_value_unset(&value);
             } else {
