@@ -114,6 +114,7 @@ struct _XfceDesktopPriv
     gboolean icons_font_size_set;
     guint icons_font_size;
     guint icons_size;
+    gint  style_refresh_timer;
     GtkWidget *icon_view;
     gdouble system_font_size;
 #endif
@@ -533,6 +534,12 @@ screen_size_changed_cb(GdkScreen *gscreen, gpointer user_data)
     if(current_workspace < 0)
         return;
 
+    /* release the bg_pixmap since the dimensions may have changed */
+    if(desktop->priv->bg_pixmap) {
+        g_object_unref(desktop->priv->bg_pixmap);
+        desktop->priv->bg_pixmap = NULL;
+    }
+
     /* special case for 1 backdrop to handle xinerama stretching */
     if(xfce_workspace_get_xinerama_stretch(desktop->priv->workspaces[current_workspace])) {
        backdrop_changed_cb(xfce_workspace_get_backdrop(desktop->priv->workspaces[current_workspace], 0), desktop);
@@ -597,7 +604,7 @@ workspace_changed_cb(WnckScreen *wnck_screen,
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
     gint current_workspace, new_workspace, i;
-    XfceBackdrop *current_backdrop, *new_backdrop;
+    XfceBackdrop *new_backdrop;
 
     TRACE("entering");
 
@@ -618,16 +625,11 @@ workspace_changed_cb(WnckScreen *wnck_screen,
         current_workspace, new_workspace);
 
     for(i = 0; i < xfce_desktop_get_n_monitors(desktop); i++) {
-        /* We want to compare the current workspace backdrop with the new one
-         * and see if we can avoid changing them if they are the same image/style */
+        /* Sanity check */
         if(current_workspace < desktop->priv->nworkspaces && current_workspace >= 0) {
-            current_backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[current_workspace], i);
+            /* update! */
             new_backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[new_workspace], i);
-
-            if(!xfce_backdrop_compare_backdrops(current_backdrop, new_backdrop) || !desktop->priv->bg_pixmap) {
-                /* only update monitors that require it */
-                backdrop_changed_cb(new_backdrop, user_data);
-            }
+            backdrop_changed_cb(new_backdrop, user_data);
         } else {
             /* If current_workspace was removed or never existed, get the new
              * backdrop and apply it */
@@ -889,6 +891,9 @@ xfce_desktop_finalize(GObject *object)
     g_object_unref(G_OBJECT(desktop->priv->channel));
     g_free(desktop->priv->property_prefix);
 
+    if(desktop->priv->style_refresh_timer != 0)
+        g_source_remove(desktop->priv->style_refresh_timer);
+
     G_OBJECT_CLASS(xfce_desktop_parent_class)->finalize(object);
 }
 
@@ -1248,28 +1253,26 @@ xfce_desktop_delete_event(GtkWidget *w,
     return TRUE;
 }
 
-static void
-xfce_desktop_style_set(GtkWidget *w,
-                       GtkStyle *old_style)
+static gboolean
+style_refresh_cb(gpointer *w)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(w);
-#ifdef ENABLE_DESKTOP_ICONS
-    gdouble old_font_size;
-#endif
-    
-    if(GDK_IS_WINDOW(desktop->priv->bg_pixmap))
-        gdk_window_set_back_pixmap(gtk_widget_get_window(w), desktop->priv->bg_pixmap, FALSE);
-    gtk_widget_queue_draw(w);
 
-#ifdef ENABLE_DESKTOP_ICONS
-    old_font_size = desktop->priv->system_font_size;
-    if(xfce_desktop_ensure_system_font_size(desktop) != old_font_size
-       && desktop->priv->icon_view && !desktop->priv->icons_font_size_set)
-    {
-        xfdesktop_icon_view_set_font_size(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
-                                          desktop->priv->system_font_size);
-    }
-#endif
+    xfce_desktop_refresh(desktop);
+    desktop->priv->style_refresh_timer = 0;
+
+    return FALSE;
+}
+
+static void
+xfce_desktop_style_set(GtkWidget *w, GtkStyle *old_style)
+{
+    XfceDesktop *desktop = XFCE_DESKTOP(w);
+
+    if(desktop->priv->style_refresh_timer != 0)
+        g_source_remove(desktop->priv->style_refresh_timer);
+
+    desktop->priv->style_refresh_timer = g_timeout_add_seconds(1, (GSourceFunc)style_refresh_cb, w);
 }
 
 static void
