@@ -31,6 +31,7 @@
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <exo/exo.h>
 
 #include "xfdesktop-app-menu-item.h"
 
@@ -45,7 +46,7 @@ struct _XfdesktopAppMenuItem
 
 typedef struct _XfdesktopAppMenuItemClass
 {
-	GtkImageMenuItemClass parent;
+    GtkImageMenuItemClass parent;
 } XfdesktopAppMenuItemClass;
 
 enum
@@ -212,6 +213,11 @@ xfdesktop_app_menu_item_set_icon(XfdesktopAppMenuItem *app_menu_item)
 
             /* Turn the pixbuf into a gtk_image */
             if(G_LIKELY(pixbuf)) {
+                /* scale the pixbuf down if it needs it */
+                GdkPixbuf *tmp = exo_gdk_pixbuf_scale_down(pixbuf, TRUE, w, h);
+                g_object_unref(pixbuf);
+                pixbuf = tmp;
+
                 image = gtk_image_new_from_pixbuf(pixbuf);
                 g_object_unref(G_OBJECT(pixbuf));
             }
@@ -228,9 +234,6 @@ static void
 xfdesktop_app_menu_item_changed(XfdesktopAppMenuItem *app_menu_item)
 {
     const gchar *label;
-#if !GTK_CHECK_VERSION (2, 16, 0)
-    GtkWidget   *child;
-#endif
 
     g_return_if_fail(XFCE_IS_APP_MENU_ITEM(app_menu_item));
     g_return_if_fail(GARCON_IS_MENU_ITEM(app_menu_item->item));
@@ -242,21 +245,7 @@ xfdesktop_app_menu_item_changed(XfdesktopAppMenuItem *app_menu_item)
     if (G_UNLIKELY (label == NULL))
       label = "";
 
-#if GTK_CHECK_VERSION(2, 16, 0)
     gtk_menu_item_set_label(GTK_MENU_ITEM(app_menu_item), label);
-#else
-    child = gtk_bin_get_child(GTK_BIN (app_menu_item));
-    if (child == NULL) {
-        child = gtk_accel_label_new(label);
-        gtk_container_add(GTK_CONTAINER(app_menu_item), child);
-        gtk_misc_set_alignment(GTK_MISC(child), 0.0, 0.5);
-        gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(child), GTK_WIDGET(app_menu_item));
-        gtk_widget_show(child);
-    } else {
-        g_return_if_fail(GTK_IS_LABEL(child));
-        gtk_label_set_text(GTK_LABEL(child), label);
-    }
-#endif
 }
 
 static void
